@@ -27,7 +27,6 @@ License along with this library.  If not, see <http://www.gnu.org/licenses/>.
 #include "../../src/client/registry.h"
 #include "../../src/client/surface.h"
 #include "../../src/client/plasmawindowmanagement.h"
-#include "../../src/client/surface.h"
 #include "../../src/server/display.h"
 #include "../../src/server/compositor_interface.h"
 #include "../../src/server/region_interface.h"
@@ -61,12 +60,15 @@ private Q_SLOTS:
     void testRequests();
     void testRequestsBoolean_data();
     void testRequestsBoolean();
+    void testKeepAbove();
+    void testKeepBelow();
     void testShowingDesktop();
     void testRequestShowingDesktop_data();
     void testRequestShowingDesktop();
     void testParentWindow();
     void testGeometry();
     void testIcon();
+    void testPid();
 
     void cleanup();
 
@@ -160,6 +162,7 @@ void TestWindowManagement::init()
     QSignalSpy windowSpy(m_windowManagement, SIGNAL(windowCreated(KWayland::Client::PlasmaWindow *)));
     QVERIFY(windowSpy.isValid());
     m_windowInterface = m_windowManagementInterface->createWindow(this);
+    m_windowInterface->setPid(1337);
 
     QVERIFY(windowSpy.wait());
     m_window = windowSpy.first().first().value<KWayland::Client::PlasmaWindow *>();
@@ -403,6 +406,7 @@ void TestWindowManagement::testRequestsBoolean_data()
     QTest::newRow("maximizable") << &PlasmaWindowInterface::maximizeableRequested << int(ORG_KDE_PLASMA_WINDOW_MANAGEMENT_STATE_MAXIMIZABLE);
     QTest::newRow("fullscreenable") << &PlasmaWindowInterface::fullscreenableRequested << int(ORG_KDE_PLASMA_WINDOW_MANAGEMENT_STATE_FULLSCREENABLE);
     QTest::newRow("skiptaskbar") << &PlasmaWindowInterface::skipTaskbarRequested << int(ORG_KDE_PLASMA_WINDOW_MANAGEMENT_STATE_SKIPTASKBAR);
+    QTest::newRow("skipSwitcher") << &PlasmaWindowInterface::skipSwitcherRequested << int(ORG_KDE_PLASMA_WINDOW_MANAGEMENT_STATE_SKIPSWITCHER);
     QTest::newRow("shadeable") << &PlasmaWindowInterface::shadeableRequested << int(ORG_KDE_PLASMA_WINDOW_MANAGEMENT_STATE_SHADEABLE);
     QTest::newRow("shaded") << &PlasmaWindowInterface::shadedRequested << int(ORG_KDE_PLASMA_WINDOW_MANAGEMENT_STATE_SHADED);
     QTest::newRow("movable") << &PlasmaWindowInterface::movableRequested << int(ORG_KDE_PLASMA_WINDOW_MANAGEMENT_STATE_MOVABLE);
@@ -475,6 +479,50 @@ void TestWindowManagement::testRequestShowingDesktop()
     QTEST(requestSpy.first().first().value<PlasmaWindowManagementInterface::ShowingDesktopState>(), "expectedValue");
 }
 
+void TestWindowManagement::testKeepAbove()
+{
+    using namespace KWayland::Server;
+    // this test verifies setting the keep above state
+    QVERIFY(!m_window->isKeepAbove());
+    QSignalSpy keepAboveChangedSpy(m_window, &KWayland::Client::PlasmaWindow::keepAboveChanged);
+    QVERIFY(keepAboveChangedSpy.isValid());
+    m_windowInterface->setKeepAbove(true);
+    QVERIFY(keepAboveChangedSpy.wait());
+    QCOMPARE(keepAboveChangedSpy.count(), 1);
+    QVERIFY(m_window->isKeepAbove());
+    // setting to same should not change
+    m_windowInterface->setKeepAbove(true);
+    QVERIFY(!keepAboveChangedSpy.wait(100));
+    QCOMPARE(keepAboveChangedSpy.count(), 1);
+    // setting to other state should change
+    m_windowInterface->setKeepAbove(false);
+    QVERIFY(keepAboveChangedSpy.wait());
+    QCOMPARE(keepAboveChangedSpy.count(), 2);
+    QVERIFY(!m_window->isKeepAbove());
+}
+
+void TestWindowManagement::testKeepBelow()
+{
+    using namespace KWayland::Server;
+    // this test verifies setting the keep below state
+    QVERIFY(!m_window->isKeepBelow());
+    QSignalSpy keepBelowChangedSpy(m_window, &KWayland::Client::PlasmaWindow::keepBelowChanged);
+    QVERIFY(keepBelowChangedSpy.isValid());
+    m_windowInterface->setKeepBelow(true);
+    QVERIFY(keepBelowChangedSpy.wait());
+    QCOMPARE(keepBelowChangedSpy.count(), 1);
+    QVERIFY(m_window->isKeepBelow());
+    // setting to same should not change
+    m_windowInterface->setKeepBelow(true);
+    QVERIFY(!keepBelowChangedSpy.wait(100));
+    QCOMPARE(keepBelowChangedSpy.count(), 1);
+    // setting to other state should change
+    m_windowInterface->setKeepBelow(false);
+    QVERIFY(keepBelowChangedSpy.wait());
+    QCOMPARE(keepBelowChangedSpy.count(), 2);
+    QVERIFY(!m_window->isKeepBelow());
+}
+
 void TestWindowManagement::testParentWindow()
 {
     using namespace KWayland::Client;
@@ -581,5 +629,22 @@ void TestWindowManagement::testIcon()
     QCOMPARE(m_window->icon().name(), QStringLiteral("xorg"));
 }
 
+void TestWindowManagement::testPid()
+{
+    using namespace KWayland::Client;
+    QVERIFY(m_window);
+    QVERIFY(m_window->pid() == 1337);
+
+    //test server not setting a PID for whatever reason
+    QScopedPointer<KWayland::Server::PlasmaWindowInterface> newWindowInterface(m_windowManagementInterface->createWindow(this));
+    QSignalSpy windowSpy(m_windowManagement, SIGNAL(windowCreated(KWayland::Client::PlasmaWindow *)));
+    QVERIFY(windowSpy.wait());
+    QScopedPointer<PlasmaWindow> newWindow( windowSpy.first().first().value<KWayland::Client::PlasmaWindow *>());
+    QVERIFY(newWindow);
+    QVERIFY(newWindow->pid() == 0);
+
+
+}
+
 QTEST_MAIN(TestWindowManagement)
 #include "test_wayland_windowmanagement.moc"
