@@ -1,5 +1,6 @@
 /********************************************************************
 Copyright 2013  Martin Gräßlin <mgraesslin@kde.org>
+Copyright 2018  Roman Gilg <subdiff@gmail.com>
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
@@ -17,8 +18,8 @@ Lesser General Public License for more details.
 You should have received a copy of the GNU Lesser General Public
 License along with this library.  If not, see <http://www.gnu.org/licenses/>.
 *********************************************************************/
-#include "logging_p.h"
 #include "outputdevice.h"
+#include "logging_p.h"
 #include "wayland_pointer_p.h"
 // Qt
 #include <QDebug>
@@ -49,7 +50,9 @@ public:
     QPoint globalPosition;
     QString manufacturer;
     QString model;
-    int scale = 1;
+    qreal scale = 1.0;
+    QString serialNumber;
+    QString eisaId;
     SubPixel subPixel = SubPixel::Unknown;
     Transform transform = Transform::Normal;
     Modes modes;
@@ -58,6 +61,9 @@ public:
     QByteArray edid;
     OutputDevice::Enablement enabled = OutputDevice::Enablement::Enabled;
     QByteArray uuid;
+
+    ColorCurves colorCurves;
+
     bool done = false;
 
 private:
@@ -67,16 +73,25 @@ private:
     static void modeCallback(void *data, org_kde_kwin_outputdevice *output, uint32_t flags, int32_t width, int32_t height, int32_t refresh, int32_t mode_id);
     static void doneCallback(void *data, org_kde_kwin_outputdevice *output);
     static void scaleCallback(void *data, org_kde_kwin_outputdevice *output, int32_t scale);
+    static void scaleFCallback(void *data, org_kde_kwin_outputdevice *output, wl_fixed_t scale);
 
     static void edidCallback(void *data, org_kde_kwin_outputdevice *output, const char *raw);
     static void enabledCallback(void *data, org_kde_kwin_outputdevice *output, int32_t enabled);
     static void uuidCallback(void *data, org_kde_kwin_outputdevice *output, const char *uuid);
 
+    static void colorcurvesCallback(void *data, org_kde_kwin_outputdevice *output,
+                                    wl_array *red, wl_array *green, wl_array *blue);
+
+    static void serialNumberCallback(void *data, org_kde_kwin_outputdevice *output, const char *serialNumber);
+    static void eisaIdCallback(void *data, org_kde_kwin_outputdevice *output, const char *eisa);
+
     void setPhysicalSize(const QSize &size);
     void setGlobalPosition(const QPoint &pos);
     void setManufacturer(const QString &manufacturer);
     void setModel(const QString &model);
-    void setScale(int scale);
+    void setScale(qreal scale);
+    void setSerialNumber(const QString &serialNumber);
+    void setEisaId(const QString &eisaId);
     void setSubPixel(SubPixel subPixel);
     void setTransform(Transform transform);
     void addMode(uint32_t flags, int32_t width, int32_t height, int32_t refresh, int32_t mode_id);
@@ -106,6 +121,14 @@ bool OutputDevice::Mode::operator==(const OutputDevice::Mode &m) const
            && output == m.output;
 }
 
+bool OutputDevice::ColorCurves::operator==(const OutputDevice::ColorCurves &cc) const
+{
+    return red == cc.red && green == cc.green && blue == cc.blue;
+}
+bool OutputDevice::ColorCurves::operator!=(const ColorCurves &cc) const {
+    return !operator==(cc);
+}
+
 OutputDevice::OutputDevice(QObject *parent)
     : QObject(parent)
     , d(new Private(this))
@@ -124,7 +147,11 @@ org_kde_kwin_outputdevice_listener OutputDevice::Private::s_outputListener = {
     scaleCallback,
     edidCallback,
     enabledCallback,
-    uuidCallback
+    uuidCallback,
+    scaleFCallback,
+    colorcurvesCallback,
+    serialNumberCallback,
+    eisaIdCallback
 };
 
 void OutputDevice::Private::geometryCallback(void *data, org_kde_kwin_outputdevice *output,
@@ -245,6 +272,13 @@ void OutputDevice::Private::scaleCallback(void *data, org_kde_kwin_outputdevice
     o->setScale(scale);
 }
 
+void OutputDevice::Private::scaleFCallback(void *data, org_kde_kwin_outputdevice *output, wl_fixed_t scale_fixed)
+{
+    auto o = reinterpret_cast<OutputDevice::Private*>(data);
+    Q_ASSERT(o->output == output);
+    o->setScale(wl_fixed_to_double(scale_fixed));
+}
+
 void OutputDevice::Private::doneCallback(void *data, org_kde_kwin_outputdevice *output)
 {
     auto o = reinterpret_cast<OutputDevice::Private*>(data);
@@ -292,6 +326,47 @@ void OutputDevice::Private::uuidCallback(void* data, org_kde_kwin_outputdevice*
     }
 }
 
+void OutputDevice::Private::colorcurvesCallback(void *data, org_kde_kwin_outputdevice *output,
+                                                wl_array *red,
+                                                wl_array *green,
+                                                wl_array *blue)
+{
+    Q_UNUSED(output);
+    auto o = reinterpret_cast<OutputDevice::Private*>(data);
+
+    auto cc = ColorCurves();
+
+    auto setCurve = [](const wl_array *curve, QVector<quint16> *destination) {
+        destination->resize(curve->size / sizeof(uint16_t));
+        memcpy(destination->data(), curve->data, curve->size);
+    };
+    setCurve(red, &cc.red);
+    setCurve(green, &cc.green);
+    setCurve(blue, &cc.blue);
+
+    if (o->colorCurves != cc) {
+        o->colorCurves = cc;
+        emit o->q->colorCurvesChanged();
+        if (o->done) {
+            emit o->q->changed();
+        }
+    }
+}
+
+void OutputDevice::Private::serialNumberCallback(void *data, org_kde_kwin_outputdevice *output, const char *raw)
+{
+    auto o = reinterpret_cast<OutputDevice::Private*>(data);
+    Q_UNUSED(output);
+    o->setSerialNumber(raw);
+}
+
+void OutputDevice::Private::eisaIdCallback(void *data, org_kde_kwin_outputdevice *output, const char *raw)
+{
+    auto o = reinterpret_cast<OutputDevice::Private*>(data);
+    Q_UNUSED(output);
+    o->setEisaId(raw);
+}
+
 void OutputDevice::setup(org_kde_kwin_outputdevice *output)
 {
     d->setup(output);
@@ -322,12 +397,22 @@ void OutputDevice::Private::setModel(const QString &m)
     model = m;
 }
 
+void OutputDevice::Private::setSerialNumber(const QString &sn)
+{
+    serialNumber = sn;
+}
+
+void OutputDevice::Private::setEisaId(const QString &e)
+{
+    eisaId = e;
+}
+
 void OutputDevice::Private::setPhysicalSize(const QSize &size)
 {
     physicalSize = size;
 }
 
-void OutputDevice::Private::setScale(int s)
+void OutputDevice::Private::setScale(qreal s)
 {
     scale = s;
 }
@@ -365,6 +450,16 @@ QString OutputDevice::model() const
     return d->model;
 }
 
+QString OutputDevice::serialNumber() const
+{
+    return d->serialNumber;
+}
+
+QString OutputDevice::eisaId() const
+{
+    return d->eisaId;
+}
+
 org_kde_kwin_outputdevice *OutputDevice::output()
 {
     return d->output;
@@ -392,10 +487,16 @@ int OutputDevice::refreshRate() const
 }
 
 int OutputDevice::scale() const
+{
+    return qRound(d->scale);
+}
+
+qreal OutputDevice::scaleF() const
 {
     return d->scale;
 }
 
+
 bool OutputDevice::isValid() const
 {
     return d->output.isValid();
@@ -439,6 +540,11 @@ QByteArray OutputDevice::uuid() const
     return d->uuid;
 }
 
+OutputDevice::ColorCurves OutputDevice::colorCurves() const
+{
+    return d->colorCurves;
+}
+
 void OutputDevice::destroy()
 {
     d->output.destroy();
